@@ -7,14 +7,12 @@ license - gnu gpl 3
 https://gnu.org/licenses/gpl-3.0.en.html
 '''
 
-from prettygit.selection import Selection
-from prettygit import options
-from pynput import keyboard
 from rich import traceback
-from rich import print
+from pynput import keyboard
+from easyselect import Selection
+import shutil as sh
 import subprocess
 import rich
-import time
 import yaml
 import sys
 import os
@@ -24,22 +22,100 @@ from dataclasses import dataclass
 rich.pretty.install()
 traceback.install(show_locals=True)
 
-from prettygit import setup
+from prettygit.setup import version, yes_or_no
 
+c = rich.console.Console()
+print = c.print
+print(f'[bold][deep_sky_blue1]prettygit v[white]{version}')
 Key = keyboard.Key
-proj_dir = os.getcwd()
-config_path = Path(f'{proj_dir}/.git/easygit.yml')
+proj_path = os.getcwd()
+config_path = Path(f'{proj_path}/.git/prettygit.yml')
 run_st = subprocess.getstatusoutput
-yes_or_no = Selection(
-    items = [
-        'yes',
-        'no',
-    ],
-    styles = [
-        'green',
-        'red'
+
+
+@dataclass
+class Data:
+    branch = None
+    remote = 'origin'
+    commit_message = 'aboba'
+    git_path = None
+    config = {}
+    options_list = [
+        {
+            'args': (
+                '-h',
+                '--help'
+            ),
+            'info':
+                'show this message',
+            'examples':
+                'prettygit --help',
+        },
+        {
+            'args': (
+                '-b',
+                '--branch'
+            ),
+            'info':
+                'set branch for pushing',
+            'examples':
+                'prettygit --branch main',
+        },
+        {
+            'args': (
+                '-r',
+                '--remote'
+            ),
+            'info':
+                'set remote for pushing',
+            'examples':
+                'prettygit --remote origin',
+        },
+        {
+            'args': (
+                '-m',
+                '--message',
+                '--commit_message'
+            ),
+            'info':
+                'set commit message',
+            'examples':
+                'prettygit --commit_message aboba',
+        },
+        {
+            'args': (
+                '-g',
+                '--git_path'
+            ),
+            'info':
+                'permanently set new path for git',
+            'examples': [
+                'prettygit --git_path /bin/git',
+                'prettygit --git_path D:\\\\git\\\\git.exe',
+            ],
+        },
     ]
-)
+
+
+def create_config():
+    if config_path.exists():
+        Data.config = yml_read_file(
+            config_path
+        )
+        Data.git_path = Data.config['git_path']
+    else:
+        Data.config['branches'] = [
+            'main',
+            'beta',
+        ]
+        Data.git_path = 'git'
+        Data.config['git_path'] = Data.git_path
+
+        yml_save(
+            data = Data.config,
+            file_path = config_path,
+        )
+        git_init()
 
 
 def yml_read_str(
@@ -54,7 +130,10 @@ def yml_read_str(
 def yml_read_file(
     file_path: str | Path
 ):
-    with open(file_path, 'r') as file:
+    with open(
+        file_path,
+        'r',
+    ) as file:
         return yml_read_str(
             file
         )
@@ -73,7 +152,10 @@ def yml_save(
     data: any,
     file_path: str | Path,
 ) -> None:
-    with open(file_path, 'w') as file:
+    with open(
+        file_path,
+        'w'
+    ) as file:
         file.write(
             to_yml_str(
                 data,
@@ -82,40 +164,23 @@ def yml_save(
 
 
 def git_init():
-    check_git()
-    check_remote()
-    if config_path.exists():
-        Data.config = yml_read_file(
-            config_path
+    print(f'try init git in [blue]{proj_path}[/blue]?')
+    if yes_or_no.choose() == 'no':
+        sys.exit()
+    check_gitignore()
+    run(f'{Data.git_path} init')
+    config_path.parent.mkdir(
+        parents=True,
+        exist_ok=True,
+    )
+    check_username()
+    check_email()
+    if run(
+        f'{Data.git_path} config --global credential.helper'
+    ) != 'store':
+        run(
+            f'{Data.git_path} config --global credential.helper store'
         )
-    else:
-        print(f'try init git in [blue]{proj_dir}[/blue]?')
-        if yes_or_no.choose() == 'no':
-            sys.exit()
-        run(f'{Data.git_path} init')
-        config_path.parent.mkdir(
-            parents=True,
-            exist_ok=True,
-        )
-        Data.config = {
-            'branches': [
-                'main',
-            ],
-        }
-        yml_save(
-            data = Data.config,
-            file_path = config_path,
-        )
-        check_git()
-        check_username()
-        check_email()
-        check_gitignore()
-        if run(
-            f'{Data.git_path} config --global credential.helper'
-        ) != 'store':
-            run(
-                f'{Data.git_path} config --global credential.helper store'
-            )
 
 
 def run(
@@ -126,42 +191,6 @@ def run(
     )[-1]
 
 
-def set_git_path(
-    *_,
-    arg_pos,
-    **__,
-):
-    Data.git_path = sys.argv[arg_pos + 1]
-    print(f'set git path "{Data.git_path}"')
-
-
-def set_branch(
-    *_,
-    arg_pos,
-    **__,
-):
-    Data.branch = sys.argv[arg_pos + 1]
-    print(f'set branch "{Data.branch}"')
-
-
-def set_remote(
-    *_,
-    arg_pos,
-    **__,
-):
-    Data.remote = sys.argv[arg_pos + 1]
-    print(f'set remote "{Data.remote}"')
-
-
-def set_commit_message(
-    *_,
-    arg_pos,
-    **__,
-):
-    Data.commit_message = sys.argv[arg_pos + 1]
-    print(f'set commit_message "{Data.commit_message}"')
-
-
 def check_username():
     if not run(
         f'{Data.git_path} config --global user.name'
@@ -201,11 +230,42 @@ def check_email():
 
 
 def check_git():
-    if run(
-        f'{Data.git_path} -v'
-    ).split()[0] != 'git':
-        print('[red]git is not installed, please install it')
-        exit()
+    selection = None
+    while True:
+        if run(
+            f'{Data.git_path} --version'
+        ).split()[0] == 'git':
+            return
+        else:
+            if not selection:
+                selection = Selection(
+                    items = [
+                        'try again',
+                        'input path',
+                        'exit'
+                    ],
+                    styles = [
+                        'green',
+                        'blue',
+                        'red',
+                    ],
+                )
+            if Data.git_path == 'git':
+                print('[red]can\'t find git on this computer')
+            else:
+                print(f'[red]can\'t find git on this path - "{Data.git_path}"')
+            answer = selection.choose()
+            match answer:
+                case 'exit':
+                    exit()
+                case 'input path':
+                    print('[blue]please input path for git:')
+                    Data.git_path = input()
+                    Data.config['git_path'] = Data.git_path
+                    yml_save(
+                        data = Data.config,
+                        file_path = config_path,
+                    )
 
 
 def inp(
@@ -286,7 +346,7 @@ def check_remote(
 
 def check_gitignore():
     gitignore_path = Path(
-        f"{proj_dir}/.gitignore"
+        f"{proj_path}/.gitignore"
     )
     if not gitignore_path.exists():
         with open(gitignore_path, 'w') as gitignore:
@@ -294,6 +354,7 @@ def check_gitignore():
 '''\
 __pycache__
 test.*
+dist
 '''
             )
 
@@ -334,15 +395,17 @@ def select_branch():
         ] + [
             'add new',
             'delete branch',
+            'cancel',
         ]
         branches = Selection(
             branches_list
         )
-        branches.styles[-2:] = (
+        branches.styles[-3:] = (
             'green',
             'red',
+            'bright_black',
         )
-        print('[green]select branch:')
+        print('\n[green]select branch:')
         branch = branches.choose()
         match branch:
             case 'add new':
@@ -357,98 +420,145 @@ def select_branch():
                 )
             case 'delete branch':
                 delete_branch()
+            case 'cancel':
+                exit()
             case _:
                 return branch
 
 
-@dataclass
-class Data:
-    branch = None
-    remote = 'origin'
-    commit_message = 'aboba'
-    git_path = "git"
-    config = {}
-    options_list = [
-        {
-            'args': (
-                '-h',
-                '--help'
-            ),
-            'info':
-                'show this message',
-            'examples':
-                'easygit --help',
-            'run':
-                options.get_help,
-            'skip_next': False
-        },
-        {
-            'args': (
-                '-b',
-                '--branch'
-            ),
-            'info':
-                'set branch for pushing',
-            'examples':
-                'easygit --branch main',
-            'run':
-                set_branch,
-            'skip_next': True
-        },
-        {
-            'args': (
-                '-r',
-                '--remote'
-            ),
-            'info':
-                'set remote for pushing',
-            'examples':
-                'easygit --remote origin',
-            'run':
-                set_remote,
-            'skip_next': True
-        },
-        {
-            'args': (
-                '-m',
-                '--commit_message'
-            ),
-            'info':
-                'set commit message',
-            'examples':
-                'easygit --commit_message aboba',
-            'run':
-                set_commit_message,
-            'skip_next': True
-        },
-        {
-            'args': (
-                '-g',
-                '--git_path'
-            ),
-            'info':
-                'set path for git',
-            'examples': [
-                'easygit --git_path /bin/git',
-                'easygit --git_path D:\\\\git\\\\git.exe',
-            ],
-            'run':
-                set_git_path,
-            'skip_next': True
-        },
-    ]
+def get_help(
+    *_,
+    options_list,
+    **__
+):
+    def rule():
+        c.rule(
+            '[bold blue]Help',
+            style = 'green',
+            characters = '━'
+        )
+    rule()
+    table = rich.table.Table(
+        show_header = False,
+        show_edge = False,
+        expand = True,
+        border_style = 'blue',
+    )
+    table.add_column()
+
+    for option in options_list:
+        text = f'''\
+[light_slate_blue]    args:  [white]\
+{' [bright_black]|[/bright_black] '.join(option['args'])}
+[light_slate_blue]    info:  [purple]\
+{option['info']}
+'''
+        if isinstance(
+            option['examples'],
+            list
+        ):
+            text += '[light_slate_blue]examples:  [medium_purple2]'
+            text += ("\n" + " " * 11).join(option['examples'])
+        else:
+            text += (
+f'[light_slate_blue] example:  [medium_purple2]{option["examples"]}'
+            )
+        table.add_row(
+            text,
+            end_section = True,
+        )
+    print(table)
+    rule()
+    print(f'[bold green] PrettyGit [bold blue]v{version}')
+
+
+def parse_args(
+    options_list,
+):
+    def check_arg(
+        name,
+        arg,
+    ):
+        if index >= len(sys.argv):
+            print(
+                f'[bold red]excepted {name} after [bold blue]{arg}[/bold blue] argument but nothing got'
+            )
+            exit()
+
+    index = 1
+
+    for arg in sys.argv[1:]:
+        index += 1
+
+        match arg:
+            case 'h' | '-h' | 'help' | '--help':
+                get_help(
+                    options_list = Data.options_list
+                )
+                exit()
+
+            case '-b' | '--branch':
+                check_arg('branch name', arg)
+                Data.branch = sys.argv[index]
+                print(f'set branch "{Data.branch}"')
+
+            case '-r' | '--remote':
+                check_arg('remote name', arg)
+                Data.remote = sys.argv[index]
+                print(f'set remote "{Data.remote}"')
+
+            case '-m' | '--message' | '--commit_message':
+                check_arg('commit_message', arg)
+                Data.commit_message = sys.argv[index]
+                print(f'set commit_message "{Data.commit_message}"')
+
+            case '-g' | '--git_path':
+                check_arg('git path', arg)
+                Data.git_path = sys.argv[index]
+                Data.config['git_path'] = Data.git_path
+                yml_save(
+                    data = Data.config,
+                    file_path = config_path,
+                )
+                print(f'permanently set git path "{Data.git_path}"')
+
+            case _:
+                pass
+
+
+def pypi():
+    if not Path(
+        f'{proj_path}/pyproject.toml'
+    ).exists():
+        return
+    print('do you want to upload package to pypi?')
+    if yes_or_no.choose() == 'no':
+        return
+    dist_path = Path(f'{proj_path}/dist')
+    print(f'remove [deep_sky_blue1]{dist_path}[/deep_sky_blue1]?')
+    if yes_or_no.choose() == 'no':
+        return
+    sh.rmtree(
+        dist_path,
+        ignore_errors=True,
+    )
+    os.system(f'{sys.executable} -m hatchling build')
+    os.system(f'{sys.executable} -m twine upload dist/*')
 
 
 def main():
+    parse_args(Data.options_list)
+    create_config()
+    check_git()
+    check_remote()
     check_gitignore()
-    options.parse(Data.options_list)
-    git_init()
-    os.system(f'{Data.git_path} add --all')
-    os.system(f'{Data.git_path} commit -m "{Data.commit_message}"')
     if not Data.branch:
         Data.branch = select_branch()
+    os.system(f'{Data.git_path} add --all')
+    os.system(f'{Data.git_path} commit -m "{Data.commit_message}"')
     run(f'{Data.git_path} branch -m {Data.branch}')
     os.system(f'{Data.git_path} push --set-upstream {Data.remote} {Data.branch}')
+    pypi()
 
 
 main()
